#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file ListnerHolder.py
# @brief Listener holder class
# @date $Date$
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2011
#     Noriaki Ando
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.
#
# $Id$
#

import threading
import OpenRTM_aist


##
# @if jp
# @class Listener ۥ饹
#
# Υ饹ϡꥹʥ饹ñݻԤꥹʥۥ
# Ǥ롣Υ饹Ѥ뤿ˤϡƥץ졼Ȥ裱
# ꥹʥ饹 (Listener饹) ӡListenerHolder饹
# ƥץ졼ȤѾơºݤ˥ꥹʤθƤӽФԤ
# ListenerHolder饹ɬפ롣
#
# Υ饹ϡåɥ֤¸뤿ᡢꥹʤɲäȺˤ
# Ƥϥߥ塼ƥåˤåԤäƤ롣˥åɥ
# ʥꥹʴ¸뤿ˤϥꥹʤΥХå򥳡뤹ݤ
# ߥ塼ƥåˤåԤɬפ롣
#
# @section Listener饹
#
# ٥ȯ˥ХåдؿĴ쥯饹
# 롣ХåΤΥдؿϡǤդ͡Ĥ
# ΤǤ̾δؿǤäƤ褤operator()ʤɤΥե
# ȤƤ褤ºݤˤϴ쥯饹ˤƤδؿ貾
# ۴ؿȤΥ饹ѾơºݤΥꥹʥ饹
# 뤳Ȥˤʤ롣ޤҤȤĤΥꥹʥ饹ʣΥХåؿ
# Ƥ褤ºݤˤϡΥХåؿºݤ˸Ƥӽ
# ˡ˴ؤƤϡListenerHolder饹ˤƾܤ뤳
# Ȥˤʤ롣
# <pre>
# class MyListenerBase
# {
# public:
#   // Хåؿ1: ؿƤӽФ黻Ҥˤ륳Хåؿ
#   // ե󥯥Τ褦˥Хåؿ㡣
#   virtual void operator()(std::string strarg) = 0; // 貾۴ؿ
#   
#   // Хåδؿ˥㤬¿ͤǤ硢Τ褦ñ
#   // дؿȤ뤳Ȥǽ
#   virtual void onEvent0(const char* arg0) = 0;
#   virtual void onEvent1(int arg0) = 0;
#   virtual void onEvent2(double arg0) = 0;
#   virtual void onEvent3(HogeProfile& arg0) = 0;
# };
# </pre>
#
# @section ListenerHolder饹
#
# ListenerHolder饹ϤLsitenerHolder饹ƥץ졼Ȥ
# ơ MyListenerBase 饹ɲäȺʤɴ
# ļºݤ˥ХåؿƤӽФʬ뤳Ȥˤʤ롣
# ºݤ˥ХåƤӽФʬǤϡؿ˥㤬¿¿ͤǤ
# ꡢҤȤĤΥꥹʥ饹ʣΥХåؿľ礬
# ᡢ̤Υꥹʥ饹б뤿ᡢθƤӽФʬɬפȤ
# 롣ListenerHolder饹ϡMyListenerBase饹Ʊ˥
# ĥдؿؿǤϡListenerHolder饹ġ
# m_listeners, m_mutex ΤĤΥѿѤơϿ
# ꥹʥ֥ȤΥѿƤӽФ
#
# <pre>
# class MyListenerHolderImpl
#  : public ::RTM::util::ListenerHolder<MyListenerBase>
# {
# public:
#   // ؿƤӽФ黻ҤΥХåؿξ
#   virtual void operator()(std::string strarg)
#   {
#     Gurad gurad(m_mutex);
#     for (int i(0), len(m_listeners.size()); i < len; ++i)
#     {
#       m_listeners[i].first->operator()(strarg);
#     }
#   }
#
#   virtual void onEvent0(const char* arg0)
#   {
#     Gurad gurad(m_mutex);
#     for (int i(0), len(m_listeners.size()); i < len; ++i)
#     {
#       m_listeners[i].first->onEvent(arg0);
#     }
#   }
# };
# </pre>
#
# ꥹʥ֥ȤؤΥݥ󥿤ǼƤEntry֥Ȥ
# std::pair<ListenerClass, bool> ȤƤꡢfirst
# Listener֥ȤؤΥݥ󥿡secondưե饰Ǥ롣
# äơꥹʥ֥ȤإˤfirstѤ롣
# ޥåɴĶѤ뤳ȤꤵϡGuard
# guard(m_mutex) ˤå˺줺˹Ԥȡ
# 
# @section ListenerHolder饹
# 줿MyListenerHolderImplϰȤưʲΤ褦Ѥ롣
#
# <pre>
# // ȤХ饹ФȤ
# MyListenerHolderImpl m_holder;
#
# // Ͽư꡼⡼ɤϿ
# // ֥ȤκHolder饹Ǥ
# m_holder.addListener(new MyListener0(), true); // MyListener0
# 
# // ХåƤӽФ
# m_holder.operator()(strarg);
# m_holder.onEvent0("HogeHoge);
# </pre>
#
# @else
#
# @class Listener holder class
#
# @endif
#
class ListenerHolder:
  """
  """

  ##
  # @if jp
  # @brief ListenerHolder饹󥹥ȥ饯
  # @else
  # @brief ListenerHolder class ctor 
  # @endif
  def __init__(self):
    self.listener_mutex = threading.RLock()
    self.listeners = []
    return


  ##
  # @if jp
  # @brief ListenerHolderǥȥ饯
  # @else
  # @brief ListenerHolder class dtor 
  # @endif
  def __del__(self):
    guard = OpenRTM_aist.ScopedLock(self.listener_mutex)
    
    for listener_ in self.listeners:
      for (l,f) in listener_.iteritems():
        if f:
          del l
    del guard
    return
  
  ##
  # @if jp
  # @brief ꥹʤɲä
  # @else
  # @brief add listener object
  # @endif
  # virtual void addListener(ListenerClass* listener,
  #                          bool autoclean)
  def addListener(self, listener, autoclean):
    guard = OpenRTM_aist.ScopedLock(self.listener_mutex)
    self.listeners.append({listener:autoclean})
    del guard
    return
    
  ##
  # @if jp
  # @brief ꥹʤ
  # @else
  # @brief remove listener object
  # @endif
  # virtual void removeListener(ListenerClass* listener)
  def removeListener(self, listener):
    guard = OpenRTM_aist.ScopedLock(self.listener_mutex)
    for (i, listener_) in enumerate(self.listeners):
      if listener == listener:
        del self.listeners[i]
        return
    return

  def LISTENERHOLDER_CALLBACK(self, func, *args):
    guard = OpenRTM_aist.ScopedLock(self.listener_mutex)
    for listener in self.listeners:
      for (l,f) in listener.iteritems():
        func_ = getattr(l,func,None)
        func_(*args)
    return
